@@ -1,11 +1,11 @@
 .onLoad <- function(lib, pkg) {
-    pkgList <- c("stashR", "digest")
+    pkgList <- c("digest", "filehash", "stashR")
     
     for(pkg in pkgList) {
         if(!require(pkg, quietly = TRUE, character.only = TRUE))
             stop(gettextf("'%s' package required", pkg))
     }
-    assign("cacheDir", ".", cacheEnv)
+    assign("cacheDir", ".", .cacheEnv)
 }
 
-cacheEnv <- new.env(parent = emptyenv())
+.cacheEnv <- new.env(parent = emptyenv())
